#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the status view CLI arguments helper."""

import sys
import unittest

from plaso.cli import status_view as cli_status_view
from plaso.cli import tools
from plaso.cli.helpers import status_view
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class StatusViewArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the status view CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _PYTHON3_13_OR_LATER = sys.version_info[0:2] >= (3, 13)

  if _PYTHON3_13_OR_LATER:
    _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--status_view TYPE] [--status_view_file PATH]
                     [--status_view_interval SECONDS]

Test argument parser.

{0:s}:
  --status_view, --status-view TYPE
                        The processing status view mode: "file", "linear",
                        "none" or "window".
  --status_view_file, --status-view-file PATH
                        The name of the status view file.
  --status_view_interval, --status-view-interval SECONDS
                        Number of seconds to update the status view.
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  else:
    _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--status_view TYPE] [--status_view_file PATH]
                     [--status_view_interval SECONDS]

Test argument parser.

{0:s}:
  --status_view TYPE, --status-view TYPE
                        The processing status view mode: "file", "linear",
                        "none" or "window".
  --status_view_file PATH, --status-view-file PATH
                        The name of the status view file.
  --status_view_interval SECONDS, --status-view-interval SECONDS
                        Number of seconds to update the status view.
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = self._GetTestArgumentParser('cli_helper.py')

    status_view.StatusViewArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.status_view_mode = cli_status_view.StatusView.MODE_WINDOW
    options.status_view_interval = 0.5

    test_tool = tools.CLITool()
    status_view.StatusViewArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._status_view_mode, options.status_view_mode)

    with self.assertRaises(errors.BadConfigObject):
      status_view.StatusViewArgumentsHelper.ParseOptions(options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the engine."""

import os
import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.path import path_spec
from dfvfs.resolver import context
from dfvfs.vfs import file_system as dfvfs_file_system

from plaso.containers import artifacts as containers_artifacts
from plaso.engine import configurations
from plaso.engine import engine
from plaso.lib import errors
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib


class TestEngine(engine.BaseEngine):
  """Class that defines the processing engine for testing."""

  def __init__(self):
    """Initialize a test engine object."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    test_file_path = shared_test_lib.GetTestFilePath(['SOFTWARE'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SOFTWARE', test_file_path)
    test_file_path = shared_test_lib.GetTestFilePath(['SYSTEM'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SYSTEM', test_file_path)

    super(TestEngine, self).__init__()
    self._file_system = file_system_builder.file_system
    self._mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

  def GetSourceFileSystem(self, file_system_path_spec, resolver_context=None):
    """Retrieves the file system of the source.

    Args:
      file_system_path_spec (dfvfs.PathSpec): path specifications of
          the source file system to process.
      resolver_context (dfvfs.Context): resolver context.

    Returns:
      tuple: containing:

        dfvfs.FileSystem: file system
        path.PathSpec: mount point path specification. The mount point path
            specification refers to either a directory or a volume on a storage
            media device or image. It is needed by the dfVFS file system
            searcher (FileSystemSearcher) to indicate the base location of
            the file system
    """
    return self._file_system, self._mount_point


class BaseEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the engine object."""

  # pylint: disable=protected-access

  def testStartStopProfiling(self):
    """Tests the _StartProfiling and _StopProfiling functions."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.profiling.directory = temp_directory
      configuration.profiling.profilers = set([
          'memory', 'parsers', 'processing', 'serializers', 'storage',
          'task_queue'])

      test_engine = engine.BaseEngine()

      test_engine._StartProfiling(None)

      test_engine._StartProfiling(configuration.profiling)
      test_engine._StopProfiling()

  def testBuildArtifactsRegistry(self):
    """Tests the BuildArtifactsRegistry function."""
    test_artifacts_path = shared_test_lib.GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_engine = TestEngine()

    self.assertIsNone(test_engine._artifacts_registry)

    test_engine.BuildArtifactsRegistry(test_artifacts_path, None)

    self.assertIsNotNone(test_engine._artifacts_registry)

    # TODO: add test that raises BadConfigOption

  def testBuildCollectionFilters(self):
    """Tests the BuildCollectionFilters function."""
    test_artifacts_path = shared_test_lib.GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_engine = TestEngine()
    test_engine.BuildArtifactsRegistry(test_artifacts_path, None)

    # Test with artifact_filter_names
    artifact_filter_names = ['TestFiles', 'TestFiles2']
    environment_variables = [
        containers_artifacts.EnvironmentVariableArtifact(
            case_sensitive=False, name='systemdrive', value='C:'
        )
    ]
    test_user_accounts = [
        containers_artifacts.UserAccountArtifact(
            identifier='1000',
            path_separator='\\',
            user_directory='C:\\Users\\testuser1',
            username='testuser1',
        ),
        containers_artifacts.UserAccountArtifact(
            identifier='1001',
            path_separator='\\',
            user_directory='%%environ_systemdrive%%\\Users\\testuser2',
            username='testuser2',
        ),
    ]

    # Pass artifact_filter_names to CreateSession
    session = test_engine.CreateSession(
        artifact_filter_names=artifact_filter_names)
    test_engine.BuildCollectionFilters(
        environment_variables,
        test_user_accounts,
        artifact_filter_names=session.artifact_filters,
        enable_artifacts_map=True
    )

    self.assertIsNotNone(test_engine._artifacts_trie)
    # Verify content of the artifacts trie
    artifacts_trie_root_children = test_engine._artifacts_trie.root.children
    self.assertIn(os.sep, artifacts_trie_root_children)
    trie_root = artifacts_trie_root_children[os.sep]
    self.assertIn(
        'test_data',
        trie_root.children)
    self.assertIn(
        '*.evtx',
        trie_root.children['test_data'].children)
    self.assertIn(
        'Users',
        trie_root.children)
    users_child = trie_root.children['Users']
    self.assertIn(
        'testuser1',
        users_child.children)
    self.assertIn(
        'testuser2',
        users_child.children)
    testuser1_child = users_child.children['testuser1']
    testuser2_child = users_child.children['testuser2']
    self.assertIn(
        'Documents',
        testuser1_child.children)
    self.assertIn(
        'Documents',
        testuser2_child.children)
    documents_child_user1 = testuser1_child.children['Documents']
    documents_child_user2 = testuser2_child.children['Documents']
    self.assertIn(
        'WindowsPowerShell',
        documents_child_user1.children)
    self.assertIn(
        'WindowsPowerShell',
        documents_child_user2.children)
    self.assertIn(
        'profile.ps1',
        documents_child_user1.children['WindowsPowerShell'].children)
    self.assertIn(
        'profile.ps1',
        documents_child_user2.children['WindowsPowerShell'].children)

    # Test with filter_file_path
    test_filter_file_path = self._GetTestFilePath(
        ['end_to_end', 'filter_file2.yaml'])
    self._SkipIfPathNotExists(test_filter_file_path)

    test_engine.BuildCollectionFilters(
        environment_variables,
        test_user_accounts,
        filter_file_path=test_filter_file_path,
        enable_artifacts_map=True
    )

    self.assertIsNotNone(test_engine._included_file_system_find_specs)
    self.assertIsNotNone(test_engine._excluded_file_system_find_specs)

    # Test specific file paths from filter file.
    included_find_specs = test_engine.GetCollectionIncludedFindSpecs()
    self.assertGreater(len(included_find_specs), 0)

    # Test with invalid filter
    with self.assertRaises(errors.InvalidFilter):
      test_engine.BuildCollectionFilters(
          environment_variables,
          test_user_accounts,
          artifact_filter_names=['NonExistentArtifact'],
          enable_artifacts_map=True
      )

  def testCreateSession(self):
    """Tests the CreateSession function."""
    test_engine = engine.BaseEngine()

    session = test_engine.CreateSession()
    self.assertIsNotNone(session)

  def testGetSourceFileSystem(self):
    """Tests the GetSourceFileSystem function."""
    test_engine = engine.BaseEngine()

    test_file_path = self._GetTestFilePath(['ímynd.dd'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location='/',
        parent=os_path_spec)

    resolver_context = context.Context()
    (
        test_file_system,
        test_mount_point,
    ) = test_engine.GetSourceFileSystem(
        source_path_spec, resolver_context=resolver_context
    )

    self.assertIsNotNone(test_file_system)
    self.assertIsInstance(test_file_system, dfvfs_file_system.FileSystem)

    self.assertIsNotNone(test_mount_point)
    self.assertIsInstance(test_mount_point, path_spec.PathSpec)

    with self.assertRaises(RuntimeError):
      test_engine.GetSourceFileSystem(None)

  def testPreprocessSource(self):
    """Tests the PreprocessSource function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    test_artifacts_path = shared_test_lib.GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_engine = TestEngine()
    test_engine.BuildArtifactsRegistry(test_artifacts_path, None)

    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    source_configurations = test_engine.PreprocessSource(
        [source_path_spec], storage_writer
    )

    self.assertEqual(len(source_configurations), 1)
    self.assertEqual(source_configurations[0].operating_system, "Windows NT")


if __name__ == "__main__":
  unittest.main()



# -*- coding: utf-8 -*
"""Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

import unittest

from plaso.parsers.jsonl_plugins import gcp_log

from tests.parsers.jsonl_plugins import test_lib


class GCPLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = gcp_log.GCPLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['gcp_logging.jsonl'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 11)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'caller_ip': '1.1.1.1',
        'container': None,
        'event_subtype': 'beta.compute.networks.insert',
        'event_type': None,
        'filename': None,
        'firewall_rules': None,
        'firewall_source_ranges': None,
        'log_name': (
            'projects/fake-project/logs/cloudaudit.googleapis.com%2Factivity'),
        'message': None,
        'policy_deltas': None,
        'recorded_time': '2021-10-19T02:57:47.339377+00:00',
        'request_account_identifier': None,
        'request_description': None,
        'request_direction': None,
        'request_email': None,
        'request_member': None,
        'request_name': None,
        'request_target_tags': None,
        'resource_labels': [
            'network_id: 4881930676783516364', 'project_id: fake-project'],
        'resource_name': 'projects/fake-project/global/networks/test',
        'service_account_display_name': None,
        'service_name': 'compute.googleapis.com',
        'severity': 'NOTICE',
        'text_payload': None,
        'user_agent': 'UserAgent'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testComputeInstancesInsert(self):
    """Tests for the JSON-L parser plugin parsing request type
    compute.instances.insert."""
    plugin = gcp_log.GCPLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['gcp_logging.jsonl'], plugin)

    expected_event_values = {
        'caller_ip': '1.1.1.1',
        'container': None,
        'dcsa_emails': ['fake-service-account@fake-project.com'],
        'dcsa_scopes': ['https://www.googleapis.com/auth/cloud-platform'],
        'delegation_chain': (
            'service-account-one@fake-project.com->'
            'service-account-two@fake-project.com'),
        'event_subtype': 'beta.compute.instances.insert',
        'event_type': None,
        'filename': None,
        'firewall_rules': None,
        'firewall_source_ranges': None,
        'gcloud_command_identity': 'a1b2c3d4e5f6',
        'gcloud_command_partial': 'gcloud compute instances insert',
        'log_name': (
            'projects/fake-project/logs/cloudaudit.googleapis.com%2Factivity'),
        'method_name': 'beta.compute.instances.insert',
        'message': None,
        'permissions': [
            'compute.instances.create',
            'compute.disks.create',
            'compute.subnetworks.use',
            'compute.subnetworks.useExternalIp',
            'compute.instances.setMetadata',
            'compute.instances.setLabels',
            'compute.instances.setServiceAccount'],
        'policy_deltas': None, 
        'principal_email': 'fake-account@fake-project.com',
        'principal_subject': 'user:fake-account@fake-project.com',
        'recorded_time': '2024-04-26T20:10:10.024055+00:00',
        'request_account_identifier': None,
        'request_description': 'Fake GCE instance',
        'request_direction': None,
        'request_email': None,
        'request_member': None,
        'request_metadata': None,
        'request_name': 'fake-compute-instance',
        'request_target_tags': None,
        'resource_labels': [
            'instance_id: 9876543210',
            'project_id: fake-project',
            'zone: us-central1-b'],
        'resource_name': (
            'projects/1234567890/zones/us-central1-b/instances/'
            'fake-compute-instance'),
        'service_account_delegation': [
            'service-account-one@fake-project.com',
            'service-account-two@fake-project.com'],
        'service_account_display_name': None,
        'service_account_key_name': None,
        'service_name': 'compute.googleapis.com',
        'severity': 'NOTICE',
        'source_images': [
            'projects/fake-project/global/images/fake-source-image'],
        'status_code': None,
        'status_message': None,
        'text_payload': None,
        'user_agent': (
            'fake-user-agent-string command/gcloud.compute.instances.insert'
            ' invocation-id/a1b2c3d4e5f6 environment/GCE')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 9)
    self.CheckEventData(event_data, expected_event_values)

  def testServiceAccountCreateFailure(self):
    """Tests service account creation failure log."""
    plugin = gcp_log.GCPLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['gcp_logging.jsonl'], plugin)

    expected_event_values = {
        'caller_ip': '34.72.217.225',
        'container': None,
        'dcsa_emails': None,
        'dcsa_scopes': None,
        'delegation_chain': ('service-1234567890@compute-system.iam.'
            'gserviceaccount.com'),
        'event_subtype': 'google.iam.admin.v1.CreateServiceAccount',
        'event_type': None,
        'filename': None,
        'firewall_rules': None,
        'firewall_source_ranges': None,
        'gcloud_command_identifier': None,
        'gcloud_command_pattern': None,
        'log_name': ('projects/ketchup/logs/cloudaudit.googleapis.com%2F'
            'activity'),
        'message': None,
        'method_name': 'google.iam.admin.v1.CreateServiceAccount',
        'permissions': ['iam.serviceAccounts.create'],
        'policy_deltas': None,
        'principal_email': ('dvwa-service-account@ketchup.iam.'
            'gserviceaccount.com'),
        'principal_subject': ('serviceAccount:dvwa-service-account@ketchup.'
            'iam.gserviceaccount.com'),
        'recorded_time': '2024-12-03T17:58:44.882119+00:00',
        'request_account_identifier': 'theattacker',
        'request_address': None,
        'request_description': None,
        'request_direction': None,
        'request_email': None,
        'request_member': None,
        'request_metadata': None,
        'request_name': 'projects/ketchup',
        'request_target': None,
        'request_labels': None,
        'resource_name': 'projects/ketchup',
        'service_account_delegation': [
            'service-1234567890@compute-system.iam.gserviceaccount.com'],
        'service_account_display_name': 'This is the attacker account',
        'service_account_key_name': None,
        'service_name': 'iam.googleapis.com',
        'severity': 'ERROR',
        'source_images': None,
        'status_code': '7',
        'status_message': ('Permission "iam.serviceAccounts.create" denied on'
            ' resource (or it may not exist).'),
        'status_reasons': ['IAM_PERMISSION_DENIED'],
        'text_payload': None,
        'user_agent': '(gzip),gzip(gfe)',
    }

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 10)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests that all output modules are imported correctly."""

import unittest

from tests import test_lib


class OutputImportTest(test_lib.ImportCheckTestCase):
  """Tests that analysis plugin classes are imported correctly."""

  _IGNORABLE_FILES = frozenset([
      'formatting_helper.py', 'interface.py', 'logger.py', 'manager.py',
      'mediator.py', 'shared_dsv.py', 'shared_json.py', 'shared_opensearch.py',
      'text_file.py', 'winevt_rc.py'])

  def testOutputModulesImported(self):
    """Tests that all output modules are imported."""
    self._AssertFilesImportedInInit(test_lib.OUTPUT_PATH, self._IGNORABLE_FILES)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The path helper."""

import os

from dfvfs.lib import definitions as dfvfs_definitions

from plaso.engine import logger
from plaso.lib import definitions


class PathHelper(object):
  """Class that implements the path helper."""

  _RECURSIVE_GLOB_LIMIT = 10

  _PATH_EXPANSIONS_PER_USERS_VARIABLE = {
      '%%users.appdata%%': [
          ['%%users.userprofile%%', 'AppData', 'Roaming'],
          ['%%users.userprofile%%', 'Application Data']],
      '%%users.localappdata%%': [
          ['%%users.userprofile%%', 'AppData', 'Local'],
          ['%%users.userprofile%%', 'Local Settings', 'Application Data']],
      '%%users.localappdata_low%%': [
          ['%%users.userprofile%%', 'AppData', 'LocalLow']],
      '%%users.temp%%': [
          ['%%users.localappdata%%', 'Temp']]}
  _DIRTY_CHARACTERS = frozenset([
      '\x00', '\x01', '\x02', '\x03', '\x04', '\x05', '\x06', '\x07',
      '\x08', '\x09', '\x0a', '\x0b', '\x0c', '\x0d', '\x0e', '\x0f',
      '\x10', '\x11', '\x12', '\x13', '\x14', '\x15', '\x16', '\x17',
      '\x18', '\x19', '\x1a', '\x1b', '\x1c', '\x1d', '\x1e', '\x1f',
      os.path.sep, '!', '$', '%', '&', '*', '+', ':', ';', '<', '>',
      '?', '@', '|', '~', '\x7f'])

  @classmethod
  def _ExpandUsersHomeDirectoryPathSegments(
      cls, path_segments, path_separator, user_accounts):
    """Expands a path to contain all users home or profile directories.

    Expands the artifacts path variable "%%users.homedir%%" or
    "%%users.userprofile%%".

    Args:
      path_segments (list[str]): path segments.
      path_separator (str): path segment separator.
      user_accounts (list[UserAccountArtifact]): user accounts.

    Returns:
      list[str]: paths returned for user accounts without a drive indicator.
    """
    if not path_segments:
      return []

    user_paths = []

    first_path_segment = path_segments[0].lower()
    if first_path_segment not in ('%%users.homedir%%', '%%users.userprofile%%'):
      if cls._IsWindowsDrivePathSegment(path_segments[0]):
        path_segments[0] = ''

      user_path = path_separator.join(path_segments)
      user_paths.append(user_path)

    elif user_accounts:
      for user_account in user_accounts:
        user_path_segments = user_account.GetUserDirectoryPathSegments()

        if not user_path_segments:
          continue

        if cls._IsWindowsDrivePathSegment(user_path_segments[0]):
          user_path_segments[0] = ''

        # Prevent concatenating two consecutive path segment separators.
        if not user_path_segments[-1]:
          user_path_segments.pop()

        user_path_segments.extend(path_segments[1:])

        user_path = path_separator.join(user_path_segments)
        user_paths.append(user_path)

    return user_paths

  @classmethod
  def _ExpandUsersVariablePathSegments(
      cls, path_segments, path_separator, user_accounts):
    """Expands path segments with a users variable, such as %%users.homedir%%.

    Args:
      path_segments (list[str]): path segments.
      path_separator (str): path segment separator.
      user_accounts (list[UserAccountArtifact]): user accounts.

    Returns:
      list[str]: paths for which the users variables have been expanded.
    """
    if not path_segments:
      return []

    path_segments_lower = [
        path_segment.lower() for path_segment in path_segments]

    if path_segments_lower[0] in ('%%users.homedir%%', '%%users.userprofile%%'):
      return cls._ExpandUsersHomeDirectoryPathSegments(
          path_segments, path_separator, user_accounts)

    path_expansions = cls._PATH_EXPANSIONS_PER_USERS_VARIABLE.get(
        path_segments[0], None)

    if path_expansions:
      expanded_paths = []

      for path_expansion in path_expansions:
        expanded_path_segments = list(path_expansion)
        expanded_path_segments.extend(path_segments[1:])

        paths = cls._ExpandUsersVariablePathSegments(
            expanded_path_segments, path_separator, user_accounts)
        expanded_paths.extend(paths)

      return expanded_paths

    if cls._IsWindowsDrivePathSegment(path_segments[0]):
      path_segments[0] = ''

    # TODO: add support for %%users.username%%
    path = path_separator.join(path_segments)
    return [path]

  @classmethod
  def _IsWindowsDrivePathSegment(cls, path_segment):
    """Determines if the path segment contains a Windows Drive indicator.

    A drive indicator can be a drive letter or %SystemDrive%.

    Args:
      path_segment (str): path segment.

    Returns:
      bool: True if the path segment contains a Windows Drive indicator.
    """
    if (len(path_segment) == 2 and path_segment[1] == ':' and
        path_segment[0].isalpha()):
      return True

    path_segment = path_segment.upper()
    return path_segment in ('%%ENVIRON_SYSTEMDRIVE%%', '%SYSTEMDRIVE%')

  @classmethod
  def ExpandGlobStars(cls, path, path_separator):
    """Expands globstars "**" in a path.

    A globstar "**" will recursively match all files and zero or more
    directories and subdirectories.

    By default the maximum recursion depth is 10 subdirectories, a numeric
    values after the globstar, such as "**5", can be used to define the maximum
    recursion depth.

    Args:
      path (str): path to be expanded.
      path_separator (str): path segment separator.

    Returns:
      list[str]: String path expanded for each glob.
    """
    expanded_paths = []

    path_segments = path.split(path_separator)
    last_segment_index = len(path_segments) - 1
    for segment_index, path_segment in enumerate(path_segments):
      recursion_depth = None
      if path_segment.startswith('**'):
        if len(path_segment) == 2:
          recursion_depth = 10
        else:
          try:
            recursion_depth = int(path_segment[2:], 10)
          except (TypeError, ValueError):
            logger.warning((
                f'Globstar with suffix "{path_segment:s}" in path "{path:s}" '
                f'not supported.'))

      elif '**' in path_segment:
        logger.warning((
            f'Globstar with prefix "{path_segment:s}" in path "{path:s}" not '
            f'supported.'))

      if recursion_depth is not None:
        if recursion_depth <= 1 or recursion_depth > cls._RECURSIVE_GLOB_LIMIT:
          logger.warning((
              f'Globstar "{path_segment:s}" in path "{path:s}" exceed '
              f'recursion maximum recursion depth, limiting to: '
              f'{cls._RECURSIVE_GLOB_LIMIT:d}.'))
          recursion_depth = cls._RECURSIVE_GLOB_LIMIT

        next_segment_index = segment_index + 1
        for expanded_path_segment in [
            ['*'] * depth for depth in range(1, recursion_depth + 1)]:
          expanded_path_segments = list(path_segments[:segment_index])
          expanded_path_segments.extend(expanded_path_segment)
          if next_segment_index <= last_segment_index:
            expanded_path_segments.extend(path_segments[next_segment_index:])

          expanded_path = path_separator.join(expanded_path_segments)
          expanded_paths.append(expanded_path)

    return expanded_paths or [path]

  @classmethod
  def ExpandUsersVariablePath(cls, path, path_separator, user_accounts):
    """Expands a path with a users variable, such as %%users.homedir%%.

    Args:
      path (str): path with users variable.
      path_separator (str): path segment separator.
      user_accounts (list[UserAccountArtifact]): user accounts.

    Returns:
      list[str]: paths for which the users variables have been expanded.
    """
    path_segments = path.split(path_separator)
    return cls._ExpandUsersVariablePathSegments(
        path_segments, path_separator, user_accounts)

  @classmethod
  def ExpandWindowsPath(cls, path, environment_variables):
    """Expands a Windows path containing environment variables.

    Args:
      path (str): Windows path with environment variables.
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.

    Returns:
      str: expanded Windows path.
    """
    path_segments = path.split('\\')
    path_segments = cls.ExpandWindowsPathSegments(
        path_segments, environment_variables)
    return '\\'.join(path_segments)

  @classmethod
  def ExpandWindowsPathSegments(cls, path_segments, environment_variables):
    """Expands a Windows path segments containing environment variables.

    Args:
      path_segments (list[str]): Windows path segments with environment
          variables.
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.

    Returns:
      list[str]: expanded Windows path segments.
    """
    if environment_variables is None:
      environment_variables = []

    lookup_table = {}
    if environment_variables:
      for environment_variable in environment_variables:
        attribute_name = environment_variable.name.upper()
        attribute_value = environment_variable.value
        if not isinstance(attribute_value, str):
          continue

        lookup_table[attribute_name] = attribute_value

    # Make a copy of path_segments since this loop can change it.
    for index, path_segment in enumerate(list(path_segments)):
      if (len(path_segment) <= 2 or not path_segment.startswith('%') or
          not path_segment.endswith('%')):
        continue

      path_segment_upper_case = path_segment.upper()
      if path_segment_upper_case.startswith('%%ENVIRON_'):
        lookup_key = path_segment_upper_case[10:-2]
      else:
        lookup_key = path_segment_upper_case[1:-1]
      path_segment = lookup_table.get(lookup_key, path_segment)
      path_segment = path_segment.split('\\')

      expanded_path_segments = list(path_segments[:index])
      expanded_path_segments.extend(path_segment)
      expanded_path_segments.extend(path_segments[index + 1:])

      path_segments = expanded_path_segments

    if cls._IsWindowsDrivePathSegment(path_segments[0]):
      path_segments[0] = ''

    return path_segments

  @classmethod
  def GetDisplayNameForPathSpec(cls, path_spec):
    """Retrieves the display name of a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: human readable version of the path specification or None if no path
          specification was provided.
    """
    if not path_spec:
      return None

    relative_path = cls.GetRelativePathForPathSpec(path_spec)
    if not relative_path:
      return path_spec.type_indicator

    path_type_indicator = path_spec.type_indicator

    parent_path_spec = path_spec.parent
    if parent_path_spec:
      if path_spec.type_indicator == (
          dfvfs_definitions.TYPE_INDICATOR_COMPRESSED_STREAM):
        path_type_indicator = path_spec.compression_method.upper()
        parent_path_spec = parent_path_spec.parent

      elif path_spec.type_indicator == dfvfs_definitions.TYPE_INDICATOR_GZIP:
        parent_path_spec = parent_path_spec.parent

    display_name = ':'.join([path_type_indicator, relative_path])
    display_name = display_name.translate(
        definitions.NON_PRINTABLE_CHARACTER_TRANSLATION_TABLE)

    if parent_path_spec and parent_path_spec.type_indicator == (
        dfvfs_definitions.TYPE_INDICATOR_VSHADOW):
      store_index = getattr(path_spec.parent, 'store_index', None)
      if store_index is not None:
        store_index += 1
        return f'VSS{store_index:d}:{display_name:s}'

    return display_name

  @classmethod
  def GetRelativePathForPathSpec(cls, path_spec):
    """Retrieves the relative path of a path specification.

    If a mount path is defined the path will be relative to the mount point,
    otherwise the path is relative to the root of the file system that is used
    by the path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: relative path or None.
    """
    if not path_spec:
      return None

    # TODO: Solve this differently, quite possibly inside dfVFS using mount
    # path spec.
    location = getattr(path_spec, 'location', None)
    if not location and path_spec.HasParent():
      location = getattr(path_spec.parent, 'location', None)

    if not location:
      return None

    data_stream = getattr(path_spec, 'data_stream', None)
    if data_stream:
      location = ':'.join([location, data_stream])

    location = location.translate(
        definitions.NON_PRINTABLE_CHARACTER_TRANSLATION_TABLE)

    if path_spec.type_indicator != dfvfs_definitions.TYPE_INDICATOR_OS:
      return location

    return location

  @classmethod
  def GetPathSegmentSeparator(cls, path_spec):
    """Retrieves the path segment separator of path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: path segment separator.
    """
    if path_spec:
      if path_spec.type_indicator == dfvfs_definitions.TYPE_INDICATOR_NTFS:
        return '\\'

      if path_spec.type_indicator == dfvfs_definitions.TYPE_INDICATOR_OS:
        return os.path.sep

    return '/'

  @classmethod
  def GetWindowsSystemPath(cls, path, environment_variables):
    """Retrieves a Windows system path.

    Args:
      path (str): Windows path with environment variables.
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.

    Returns:
      tuple[str, str]: Windows system path and filename.
    """
    path, _, filename = path.rpartition('\\')

    # If the path is just a filename assume the file is stored in:
    # "%SystemRoot%\System32".
    if not path:
      path = '%SystemRoot%\\System32'

    path = cls.ExpandWindowsPath(path, environment_variables)

    return path, filename

  @classmethod
  def SanitizePathSegments(cls, path_segments):
    """Sanitizes path segments.

    Replaces non-printable and other characters defined in _DIRTY_CHARACTERS
    with an underscore "_".

    Args:
      path_segments (list[str]): path segments.

    Returns:
      list[str]: sanitized path segments.
    """
    sanitized_path_segments = []
    for path_segment in path_segments:
      sanitized_path_segment = ''.join([
          character if character not in cls._DIRTY_CHARACTERS else '_'
          for character in path_segment])
      sanitized_path_segments.append(sanitized_path_segment)
    return sanitized_path_segments

  @classmethod
  def GetRelativePath(
      cls, target_directory, target_filename, destination_path):
    """Retrieves the relative path from the destination path.

    Args:
      target_directory (str): path of the target directory.
      target_filename (str): name of the target file.
      destination_path (str): destination path for the collected files.

    Returns:
      str: normalized path or None.
    """
    if not destination_path.endswith(os.path.sep):
      destination_path = destination_path + os.path.sep
    target_path = os.path.join(target_directory, target_filename)
    if target_path.startswith(destination_path):
      return target_path[len(destination_path):]
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the AMCache Registry plugin."""

import unittest

from dfwinreg import regf as dfwinreg_regf

from plaso.parsers.winreg_plugins import amcache

from tests.parsers.winreg_plugins import test_lib


class AMCachePluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the AMCache.hve Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = amcache.AMCachePlugin()

    self._AssertFiltersOnKeyPath(plugin, '', 'Root')
    self._AssertNotFiltersOnKeyPath(plugin, '', 'Root\\File')

    self._AssertNotFiltersOnKeyPath(plugin, '', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['winreg', 'Amcache.hve'])

    file_object = test_file_entry.GetFileObject()

    registry_file = dfwinreg_regf.REGFWinRegistryFile(ascii_codepage='cp1252')
    registry_file.Open(file_object)

    registry_key = registry_file.GetKeyByPath('\\Root')

    plugin = amcache.AMCachePlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3497)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_full_path = (
        'c:\\users\\user\\appdata\\local\\temp\\chocolatey\\'
        'is-f4510.tmp\\idafree50.tmp')

    expected_event_values = {
        'application_key_last_written_time': None,
        'data_type': 'windows:registry:amcache',
        'file_creation_time': '2017-08-01T12:43:35.1772758+00:00',
        'file_modification_time': '2017-08-01T12:43:35.3024523+00:00',
        'full_path': expected_full_path,
        'installation_time': None,
        'last_written_time': '2017-08-01T12:43:35.3715211+00:00',
        'link_time': '1992-06-19T22:22:17+00:00',
        'msi_installation_time': None,
        'sha1': '82274eef0911a948f91425f5e5b0e730517fe75e'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1420)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:registry:amcache:programs',
        'entry_type': 'AddRemoveProgram',
        'file_paths': [
            'c:\\program files (x86)\\fileinsight\\plugins',
            'c:\\program files (x86)\\fileinsight\\plugins\\anomaly chart',
            'c:\\program files (x86)\\fileinsight'],
        'installation_time': '2017-08-01T12:52:59+00:00',
        'name': 'FileInsight - File analysis tool',
        'publisher': 'McAfee Inc.',
        'uninstall_key': [
            'HKEY_LOCAL_MACHINE\\Software\\Wow6432Node\\Microsoft\\Windows\\'
            'CurrentVersion\\Uninstall\\FileInsight']}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3480)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWindows10(self):
    """Tests the Process function on a Windows 10 1807 AMCache.hve file."""
    test_file_entry = self._GetTestFileEntry(['winreg', 'win10-Amcache.hve'])

    file_object = test_file_entry.GetFileObject()

    registry_file = dfwinreg_regf.REGFWinRegistryFile(ascii_codepage='cp1252')
    registry_file.Open(file_object)

    registry_key = registry_file.GetKeyByPath('\\Root')

    plugin = amcache.AMCachePlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 236)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application_key_last_written_time': (
            '2019-12-17T05:30:28.2416496+00:00'),
        'data_type': 'windows:registry:amcache',
        'file_creation_time': None,
        'file_identifier': '000075c5a97f521f760e32a4a9639a653eed862e9c61',
        'file_modification_time': None,
        'file_name': 'svchost.exe',
        'full_path': 'c:\\windows\\system32\\svchost.exe',
        'installation_time': None,
        'last_written_time': None,
        'link_time': '1997-01-10T22:26:24+00:00',
        'msi_installation_time': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 135)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the artifacts trie."""

import unittest

from plaso.engine import artifacts_trie


class TrieNodeTest(unittest.TestCase):
  """Tests for the TrieNode object."""

  def test_initialization(self):
    """Test that the node can be initialized."""
    node = artifacts_trie.TrieNode()
    self.assertIsNotNone(node)
    self.assertEqual(node.children, {})
    self.assertEqual(node.artifacts_names, [])
    self.assertEqual(node.path_separator, '/')


class ArtifactsTrieTest(unittest.TestCase):
  """Tests for the artifactsTrie object."""

  def setUp(self):
    """Setup a trie for testing."""
    self.trie = artifacts_trie.ArtifactsTrie()

  def test_add_path(self):
    """Tests the AddPath function."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')
    self.trie.AddPath('artifact2', '/path/to/another.txt', '/')

    root = self.trie.root
    self.assertIn('/', root.children)
    self.assertIn('path', root.children['/'].children)
    path_children = root.children['/'].children['path'].children
    self.assertIn('to', path_children)
    to_children = path_children['to'].children
    self.assertIn(
        'file.txt', to_children)
    self.assertIn(
        'another.txt',
        to_children
    )
    self.assertEqual(
        to_children['file.txt']
        .artifacts_names,
        ['artifact1'],
    )
    self.assertEqual(
        to_children['another.txt']
        .artifacts_names,
        ['artifact2']
    )

  def test_add_path_with_glob(self):
    """Tests the AddPath function with glob patterns."""
    self.trie.AddPath('artifact1', '/path/to/*.txt', '/')
    self.trie.AddPath('artifact2', '/path/to/dir**', '/')

    root = self.trie.root
    self.assertIn('/', root.children)
    self.assertIn('path', root.children['/'].children)
    path_children = root.children['/'].children['path'].children
    self.assertIn('to', path_children)
    to_children = path_children['to'].children
    self.assertIn(
        '*.txt', to_children)
    self.assertIn(
        'dir**', to_children)
    self.assertEqual(
        to_children['*.txt']
        .artifacts_names,
        ['artifact1']
    )
    self.assertEqual(
        to_children['dir**']
        .artifacts_names,
        ['artifact2']
    )

  def test_get_matching_artifacts_no_glob(self):
    """Tests GetMatchingArtifacts without glob."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')
    self.trie.AddPath('artifact2', '/path/to/another.txt', '/')

    matches = self.trie.GetMatchingArtifacts('/path/to/file.txt', '/')
    self.assertIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

  def test_get_matching_artifacts_with_glob(self):
    """Tests GetMatchingArtifacts with glob patterns."""
    self.trie.AddPath('artifact1', '/path/to/*.txt', '/')
    self.trie.AddPath('artifact2', '/path/**/file.txt', '/')
    # The trie structure will have these paths and children
    # root
    #  |
    #  /
    #  |
    #  path
    #   |
    #  to, **
    #   |   |
    #  *.txt  file.txt

    matches = self.trie.GetMatchingArtifacts('/path/to/file.txt', '/')
    self.assertIn('artifact1', matches)
    self.assertIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts('/path/to/dir/file.txt', '/')
    self.assertIn('artifact2', matches)
    self.assertNotIn('artifact1', matches)

  def test_get_matching_artifacts_with_multiple_globs(self):
    """Tests GetMatchingArtifacts with multiple consecutive glob patterns."""
    self.trie.AddPath('artifact1', '/**/**/file.txt', '/')
    self.trie.AddPath('artifact2', '/**/**/another.txt', '/')

    # The trie structure will have these paths and children
    # root
    #  |
    #  /
    #  |
    #  **
    #   |
    #  **, file.txt, another.txt

    matches = self.trie.GetMatchingArtifacts(
        '/path/to/dir/subdir/file.txt', '/')
    self.assertIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts(
        '/path/to/dir/subdir/another.txt', '/')
    self.assertIn('artifact2', matches)
    self.assertNotIn('artifact1', matches)

  def test_get_matching_artifacts_single_asterisk(self):
    """Tests GetMatchingArtifacts with single asterisk glob patterns."""
    self.trie.AddPath('artifact1', '/path/to/*/file.txt', '/')
    self.trie.AddPath('artifact2', '/path/*/data/*.txt', '/')
    self.trie.AddPath(
        'artifact3', '/home/*/.jupyter/jupyter_notebook_config.py', '/')

    matches = self.trie.GetMatchingArtifacts('/path/to/dir/file.txt', '/')
    self.assertIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts(
        '/path/dir/data/test.txt', '/')
    self.assertNotIn('artifact1', matches)
    self.assertIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts(
        '/home/dummyuser/.jupyter/jupyter_notebook_config.py', '/')
    self.assertNotIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)
    self.assertIn('artifact3', matches)

  def test_get_matching_artifacts_windows_paths(self):
    """Tests GetMatchingArtifacts with Windows paths."""
    self.trie.AddPath('artifact1', '\\Windows\\System32\\*.dll', '\\')
    self.trie.AddPath('artifact2', '\\Users\\**\\AppData\\*', '\\')

    matches = self.trie.GetMatchingArtifacts(
        '\\Windows\\System32\\kernel32.dll', '\\')
    self.assertIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts(
        '\\Users\\test\\AppData\\Local', '\\')
    self.assertIn('artifact2', matches)
    self.assertNotIn('artifact1', matches)

  def test_get_matching_artifacts_negative_cases(self):
    """Tests GetMatchingArtifacts with non-matching cases."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')
    self.trie.AddPath('artifact2', '/path/to/*.txt', '/')
    self.trie.AddPath('artifact3', '/path/**/file.txt', '/')

    matches = self.trie.GetMatchingArtifacts('/path/to/other.txt', '/')
    self.assertNotIn('artifact1', matches)
    self.assertNotIn('artifact3', matches)

    matches = self.trie.GetMatchingArtifacts('/path/dir/file.txt', '/')
    self.assertNotIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

    matches = self.trie.GetMatchingArtifacts('/path/to/dir/test/fi*.txt', '/')
    self.assertNotIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)

  def test_add_path_with_mixed_separators(self):
    """Tests the AddPath function with mixed path separators."""
    self.trie.AddPath('artifact1', '/mixed/path/style_s', '/')
    self.trie.AddPath(
        'artifact2', '\\another\\mixed path\\style to-match_it', '\\')

    root = self.trie.root
    self.assertIn('/', root.children)
    self.assertIn('mixed', root.children['/'].children)

    self.assertIn('\\', root.children)
    self.assertIn('another', root.children['\\'].children)

  def test_get_matching_artifacts_mixed_separators(self):
    """Tests GetMatchingArtifacts with mixed path separators."""
    self.trie.AddPath('artifact1', '/mixed/path/style_s', '/')
    self.trie.AddPath(
        'artifact2', '\\another\\mixed path\\style to-match_it', '\\')

    matches = self.trie.GetMatchingArtifacts('/mixed/path/style_s', '/')
    self.assertIn('artifact1', matches)

    matches = self.trie.GetMatchingArtifacts(
        '\\another\\mixed path\\style to-match_it', '\\')
    self.assertIn('artifact2', matches)

  def test_get_matching_artifacts_same_path_different_artifacts(self):
    """Tests GetMatchingArtifacts with same path for different artifacts."""
    self.trie.AddPath('artifact1', '/same/path/file.txt', '/')
    self.trie.AddPath('artifact2', '/same/path/file.txt', '/')

    matches = self.trie.GetMatchingArtifacts('/same/path/file.txt', '/')
    self.assertIn('artifact1', matches)
    self.assertIn('artifact2', matches)

  def test_get_matching_artifacts_empty_path(self):
    """Tests GetMatchingArtifacts with an empty path."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')

    matches = self.trie.GetMatchingArtifacts('', '/')
    self.assertEqual(len(matches), 0)

  def test_get_matching_artifacts_root_path(self):
    """Tests GetMatchingArtifacts with root path."""
    self.trie.AddPath('artifact1', '/', '/')

    matches = self.trie.GetMatchingArtifacts('/', '/')
    self.assertIn('artifact1', matches)

  def test_get_matching_artifacts_nonexistent_path(self):
    """Tests GetMatchingArtifacts with a nonexistent path."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')

    matches = self.trie.GetMatchingArtifacts('/nonexistent/path', '/')
    self.assertEqual(len(matches), 0)

  def test_get_matching_artifacts_case_sensitivity(self):
    """Tests GetMatchingArtifacts with different case sensitivity."""
    self.trie.AddPath('artifact1', '/path/to/file.txt', '/')

    matches = self.trie.GetMatchingArtifacts('/Path/To/File.TXT', '/')
    self.assertNotIn('artifact1', matches)

  def test_get_matching_artifacts_special_characters(self):
    """Tests GetMatchingArtifacts with special characters in path."""
    self.trie.AddPath('artifact1', '/path/to/file[0-9].txt', '/')

    matches = self.trie.GetMatchingArtifacts('/path/to/file1.txt', '/')
    self.assertIn('artifact1', matches)

    matches = self.trie.GetMatchingArtifacts('/path/to/file.txt', '/')
    self.assertNotIn('artifact1', matches)

  def test_get_matching_artifacts_sanitized_paths(self):
    """Tests GetMatchingArtifacts with sanitized paths."""
    self.trie.AddPath('artifact1', '/path/to/file\x00\x01.txt', '/')
    self.trie.AddPath('artifact2', '/another/path/fo:r?/file.txt', '/')

    # Test path with dirty characters that would be sanitized.
    matches = self.trie.GetMatchingArtifacts(
        '/path/to/file__.txt', '/')
    self.assertIn('artifact1', matches)

    # Test with a different path that sanitizes to the same value.
    matches = self.trie.GetMatchingArtifacts(
        '/another/path/fo_r_/file.txt', '/')
    self.assertIn('artifact2', matches)

    # Negative test with non-matching sanitized path.
    matches = self.trie.GetMatchingArtifacts(
        '/nonexistent/path/fo_r_/file.txt', '/')
    self.assertNotIn('artifact1', matches)
    self.assertNotIn('artifact2', matches)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Google Chrome autofill database (Web Data) files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class ChromeAutofillEventData(events.EventData):
  """Chrome Autofill event data.

  Attributes:
    creation_time (dfdatetime.DateTimeValues): creation date and time of
        the autofill entry.
    field_name (str): name of form field.
    last_used_time (dfdatetime.DateTimeValues): last date and time
        the autofill entry was last used.
    query (str): SQL query that was used to obtain the event data.
    usage_count (int): count of times value has been used in field_name.
    value (str): value populated in form field.
  """

  DATA_TYPE = 'chrome:autofill:entry'

  def __init__(self):
    """Initializes event data."""
    super(ChromeAutofillEventData, self).__init__(data_type=self.DATA_TYPE)
    self.creation_time = None
    self.field_name = None
    self.last_used_time = None
    self.query = None
    self.usage_count = None
    self.value = None


class ChromeAutofillPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Google Chrome autofill database (Web Data) files.

  The Google Chrome autofill database (Web Data) file is typically stored in:
  Web Data
  """

  NAME = 'chrome_autofill'
  DATA_FORMAT = 'Google Chrome autofill SQLite database (Web Data) file'

  REQUIRED_STRUCTURE = {
      'autofill': frozenset([
          'date_created', 'date_last_used', 'name', 'value', 'count'])}

  QUERIES = [
      (('SELECT autofill.date_created, autofill.date_last_used, autofill.name, '
        'autofill.value, autofill.count FROM autofill ORDER BY date_created'),
       'ParseAutofillRow')]

  SCHEMAS = [{
      'autofill': (
          'CREATE TABLE autofill (name VARCHAR, value VARCHAR, '
          'value_lower VARCHAR, date_created INTEGER DEFAULT 0, '
          'date_last_used INTEGER DEFAULT 0, count INTEGER DEFAULT 1, '
          'PRIMARY KEY (name, value));)')}]

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.PosixTime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  def ParseAutofillRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses an autofill entry row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = ChromeAutofillEventData()
    event_data.creation_time = self._GetDateTimeRowValue(
        query_hash, row, 'date_created')
    event_data.field_name = self._GetRowValue(query_hash, row, 'name')
    event_data.last_used_time = self._GetDateTimeRowValue(
        query_hash, row, 'date_last_used')
    event_data.value = self._GetRowValue(query_hash, row, 'value')
    event_data.usage_count = self._GetRowValue(query_hash, row, 'count')
    event_data.query = query

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(ChromeAutofillPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the storage format CLI arguments helper."""

import sys
import unittest

from plaso.cli import tools
from plaso.cli.helpers import storage_format
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class StorageFormatArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the storage format CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _PYTHON3_13_OR_LATER = sys.version_info[0:2] >= (3, 13)

  if _PYTHON3_13_OR_LATER:
    _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--storage_format FORMAT] [--task_storage_format FORMAT]

Test argument parser.

{0:s}:
  --storage_format, --storage-format FORMAT
                        Format of the storage file, the default is: sqlite.
                        Supported options: sqlite
  --task_storage_format, --task-storage-format FORMAT
                        Format for task storage, the default is: sqlite.
                        Supported options: redis, sqlite
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  else:
    _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--storage_format FORMAT] [--task_storage_format FORMAT]

Test argument parser.

{0:s}:
  --storage_format FORMAT, --storage-format FORMAT
                        Format of the storage file, the default is: sqlite.
                        Supported options: sqlite
  --task_storage_format FORMAT, --task-storage-format FORMAT
                        Format for task storage, the default is: sqlite.
                        Supported options: redis, sqlite
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = self._GetTestArgumentParser('cli_helper.py')

    storage_format.StorageFormatArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.storage_format = 'sqlite'
    options.task_storage_format = 'sqlite'

    test_tool = tools.CLITool()
    storage_format.StorageFormatArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._storage_format, options.storage_format)
    self.assertEqual(
        test_tool._task_storage_format, options.task_storage_format)

    with self.assertRaises(errors.BadConfigObject):
      storage_format.StorageFormatArgumentsHelper.ParseOptions(options, None)

    with self.assertRaises(errors.BadConfigOption):
      options.storage_format = 'bogus'
      storage_format.StorageFormatArgumentsHelper.ParseOptions(
          options, test_tool)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""JSON-L parser plugin related functions and classes for testing."""

import json

from dfvfs.helpers import text_file

from plaso.containers import events
from plaso.parsers import mediator as parsers_mediator

from tests.parsers import test_lib


class JSONLPluginTestCase(test_lib.ParserTestCase):
  """JSON-L parser plugin test case."""

  def _ParseJSONLFileWithPlugin(self, path_segments, plugin):
    """Parses a file as an JSON-L log file and returns an event generator.

    This method will first test if a JSON-L log file has the required format
    using plugin.CheckRequiredFormat() and then extracts events using
    plugin.Process().

    Args:
      path_segments (list[str]): path segments inside the test data directory.
      plugin (JSONLPlugin): JSON-L log file plugin.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    parser_mediator = parsers_mediator.ParserMediator()

    storage_writer = self._CreateStorageWriter()
    parser_mediator.SetStorageWriter(storage_writer)

    file_entry = self._GetTestFileEntry(path_segments)
    parser_mediator.SetFileEntry(file_entry)

    if file_entry:
      event_data_stream = events.EventDataStream()
      event_data_stream.path_spec = file_entry.path_spec

      parser_mediator.ProduceEventDataStream(event_data_stream)

    # AppendToParserChain needs to be run after SetFileEntry.
    parser_mediator.AppendToParserChain('jsonl')

    file_object = file_entry.GetFileObject()
    text_file_object = text_file.TextFile(file_object)

    line = text_file_object.readline()
    json_dict = json.loads(line)

    required_format = plugin.CheckRequiredFormat(json_dict)
    self.assertTrue(required_format)

    plugin.UpdateChainAndProcess(parser_mediator, file_object=file_object)

    return storage_writer



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This file contains tests for Services Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import services

from tests.parsers.winreg_plugins import test_lib


class ServicesRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Services Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2012-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'TestDriver', key_path_prefix='HKEY_LOCAL_MACHINE\\System',
        last_written_time=filetime.timestamp, offset=1456, relative_key_path=(
            'ControlSet001\\services\\TestDriver'))

    value_data = b'\x02\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Type', data=value_data, data_type=dfwinreg_definitions.REG_DWORD,
        offset=123)
    registry_key.AddValue(registry_value)

    value_data = b'\x02\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Start', data=value_data, data_type=dfwinreg_definitions.REG_DWORD,
        offset=127)
    registry_key.AddValue(registry_value)

    value_data = b'\x01\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ErrorControl', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD, offset=131)
    registry_key.AddValue(registry_value)

    value_data = 'Pnp Filter'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Group', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=140)
    registry_key.AddValue(registry_value)

    value_data = 'Test Driver'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DisplayName', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=160)
    registry_key.AddValue(registry_value)

    value_data = 'testdriver.inf_x86_neutral_dd39b6b0a45226c4'.encode(
        'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DriverPackageId', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=180)
    registry_key.AddValue(registry_value)

    value_data = 'C:\\Dell\\testdriver.sys'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ImagePath', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=200)
    registry_key.AddValue(registry_value)

    return registry_key

  # TODO: add test for _GetServiceDll
  # TODO: add test for _GetValuesFromKey

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = services.ServicesPlugin()

    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'MRUlist', key_path_prefix='HKEY_LOCAL_MACHINE\\System',
        relative_key_path='ControlSet001\\services\\TestDriver')

    result = self._CheckFiltersOnKeyPath(plugin, registry_key)
    self.assertFalse(result)

    registry_value = dfwinreg_fake.FakeWinRegistryValue('Start')
    registry_key.AddValue(registry_value)

    registry_value = dfwinreg_fake.FakeWinRegistryValue('Type')
    registry_key.AddValue(registry_value)

    result = self._CheckFiltersOnKeyPath(plugin, registry_key)
    self.assertTrue(result)

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\System', 'Bogus')

  def testProcess(self):
    """Tests the Process function on a virtual key."""
    registry_key = self._CreateTestKey()

    plugin = services.ServicesPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_key_path = (
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\services\\TestDriver')

    expected_event_values = {
        'data_type': 'windows:registry:service',
        'error_control': 1,
        'image_path': 'C:\\Dell\\testdriver.sys',
        'key_path': expected_key_path,
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00',
        'service_type': 2,
        'start_type': 2,
        'values': [
            ('Group', 'REG_SZ', 'Pnp Filter'),
            ('DisplayName', 'REG_SZ', 'Test Driver'),
            ('DriverPackageId', 'REG_SZ',
             'testdriver.inf_x86_neutral_dd39b6b0a45226c4')]}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessFile(self):
    """Tests the Process function on a key in a file."""
    test_file_entry = self._GetTestFileEntry(['SYSTEM'])
    key_path = 'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\services\\BITS'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = services.ServicesPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:service',
        'key_path': key_path,
        'last_written_time': '2012-04-06T20:43:27.6390752+00:00',
        'service_dll': '%SystemRoot%\\System32\\qmgr.dll',
        'service_type': 0x20,
        'start_type': 3}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Motherboard Info Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import motherboard_info

from tests.parsers.winreg_plugins import test_lib


class WindowsRegistryMotherboardInfoPluginTest(
    test_lib.RegistryPluginTestCase):
  """Tests for the Motherboard Info Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2024-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
      'SystemInformation', key_path_prefix='HKEY_LOCAL_MACHINE\\System',
      last_written_time=filetime.timestamp,
      relative_key_path='ControlSet001\\Control\\SystemInformation')

    value_data = 'Microsoft Corporation'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
      'SystemManufacturer', data=value_data,
      data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'Virtual Machine'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
      'SystemProductName', data=value_data,
      data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = '05/13/2024'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
      'BIOSReleaseDate', data=value_data,
      data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'Hyper-V UEFI Release v4.1'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
      'BIOSVersion', data=value_data,
      data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcessValue(self):
    """Tests the Process function for BAM data."""
    test_file_entry = test_lib.TestFileEntry('SYSTEM')
    registry_key = self._CreateTestKey()
    plugin = motherboard_info.MotherboardInfoPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
      'bios_release_date': '05/13/2024',
      'bios_version': 'Hyper-V UEFI Release v4.1',
      'data_type': 'windows:registry:motherboard_info',
      'key_path': (
          'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\'
          'SystemInformation'),
      'last_written_time': '2024-08-28T09:23:49.0020310+00:00',
      'motherboard_manufacturer': 'Microsoft Corporation',
      'motherboard_model': 'Virtual Machine'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the parsers and plugins interface classes."""

import unittest

from plaso.parsers import interface

from tests.parsers import test_lib


class BaseParserTest(test_lib.ParserTestCase):
  """Tests for the parser interface."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = interface.BaseParser()

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    self.assertEqual(len(parser._plugins_per_name), 0)

  def testInitialize(self):
    """Tests the initialization."""
    parser = interface.BaseParser()

    self.assertIsNotNone(parser)
    self.assertIsNone(parser._default_plugin)
    self.assertEqual(parser._plugins_per_name, {})

  def testSupportsPlugins(self):
    """Tests the SupportsPlugins function."""
    self.assertFalse(interface.BaseParser.SupportsPlugins())

  # The DeregisterPlugin and RegisterPlugin functions are tested in manager.py

  # TODO: add tests for GetPluginNames
  # TODO: add tests for GetPluginObjectByName
  # TODO: add tests for GetPlugins


if __name__ == '__main__':
  unittest.main()
